/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * MoleculeOfAtomAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/MoleculeDescriptor.hpp"
#include "Descriptors/MoleculeOfAtomSelectionDescriptor.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "MoleculeOfAtomAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "MoleculeOfAtomAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionMoleculeOfAtomAction::performCall() {
  std::vector<molecule *> selectedMolecules = World::getInstance().getSelectedMolecules();
  LOG(1, "Selecting molecule to currently selected atoms: " << World::getInstance().countSelectedAtoms());
  World::getInstance().selectAllMolecules(MoleculesByAtomSelection());
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");
  return ActionState::ptr(new SelectionMoleculeOfAtomState(selectedMolecules, params));
}

ActionState::ptr SelectionMoleculeOfAtomAction::performUndo(ActionState::ptr _state) {
  SelectionMoleculeOfAtomState *state = assert_cast<SelectionMoleculeOfAtomState*>(_state.get());

  World::getInstance().clearMoleculeSelection();
  BOOST_FOREACH(molecule *_mol, state->selectedMolecules)
    World::getInstance().selectMolecule(_mol);

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionMoleculeOfAtomAction::performRedo(ActionState::ptr _state){
  //SelectionMoleculeOfAtomState *state = assert_cast<SelectionMoleculeOfAtomState*>(_state.get());

  World::getInstance().selectAllMolecules(MoleculesByAtomSelection());

  return ActionState::ptr(_state);
}

bool SelectionMoleculeOfAtomAction::canUndo() {
  return true;
}

bool SelectionMoleculeOfAtomAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
